
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.StaticFiles;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.FileProviders;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;
using Senlan.VersionApp.Data;
using Serilog;
using Serilog.Events;
using System.Reflection;
using System.Text;

var builder = WebApplication.CreateBuilder(args);

builder.Services.AddDbContext<SenlanDbContext>(options =>

    options.UseMySql(builder.Configuration.GetConnectionString("SenlanWebContext"), new MySqlServerVersion(new Version(8, 0, 27)))
           .LogTo(Console.WriteLine, LogLevel.Information)
           .EnableSensitiveDataLogging()
           .EnableDetailedErrors()
    );

Log.Logger = new LoggerConfiguration()
                .MinimumLevel.Debug()
                .MinimumLevel.Override("Microsoft", LogEventLevel.Information)
                .Enrich.FromLogContext()
                .WriteTo.File("Logs/log.txt", rollingInterval: RollingInterval.Hour)
                .CreateLogger();
builder.Host.UseSerilog();

builder.Services.AddControllers();
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddMemoryCache();

builder.Services.AddSwaggerGen(options =>
{
    options.SwaggerDoc("v1", new OpenApiInfo { Title = "APIĵ", Version = "v1" });
    // ȡxmlļ
    var xmlFile = $"{Assembly.GetExecutingAssembly().GetName().Name}.xml";
    // ȡxmlļ·
    var xmlPath = Path.Combine(AppContext.BaseDirectory, xmlFile);
    // ӿעͣtrueʾʾע
    //options.IncludeXmlComments(xmlPath, true);

    #region JWT
    options.AddSecurityRequirement(new OpenApiSecurityRequirement {
                     {
                        new OpenApiSecurityScheme
                        {
                            Reference = new OpenApiReference { Type = ReferenceType.SecurityScheme, Id = "api" }
                        },
                        new[] { "readAccess", "writeAccess" }
                    }

                });

    options.AddSecurityDefinition("api", new OpenApiSecurityScheme
    {
        Description = "ʱBearerһո",
        Name = "Authorization",
        In = ParameterLocation.Header,
        Type = SecuritySchemeType.ApiKey
    });
    #endregion
});

builder.Services.AddAuthentication(JwtBearerDefaults.AuthenticationScheme)
                     .AddJwtBearer(options =>
                     {
                         options.TokenValidationParameters = new TokenValidationParameters
                         {
                             ValidateIssuerSigningKey = true,
                             IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes("senlan123!@#$%^&*")),
                             ValidateIssuer = false,
                             ValidateAudience = false,
                             ValidateLifetime = true,
                             ClockSkew = TimeSpan.FromMinutes(5)
                         };
                     });


var types = AppDomain.CurrentDomain.GetAssemblies()
    .SelectMany(q => q.GetTypes()).Where(q => q.Name.Contains("Service")
                                           && q.FullName != null
                                           && q.FullName.Contains("Senlan")
                                           && !q.IsAbstract
                                           && !q.IsGenericType
                                           && q.IsClass)
    .ToList();

foreach (Type type in types)
{
    var interfaces = type.GetInterfaces();
    if (interfaces.Any())
    {
        var inter = interfaces[1];
        builder.Services.AddScoped(inter, type);
    }
}


var app = builder.Build();

app.UseSwagger();
app.UseSwaggerUI();
 

app.UseStaticFiles(new StaticFileOptions
{               
    ServeUnknownFileTypes = true 
});
 

app.UseCors((option) =>
{
    option.AllowAnyOrigin();
    option.AllowAnyMethod();
    option.AllowAnyHeader();
});

string pathUpload = Path.Combine(AppContext.BaseDirectory, "upload");
Directory.CreateDirectory(pathUpload);
app.UseStaticFiles(new StaticFileOptions()//ԶԼļ·
{
    RequestPath = new PathString("/upload"),//ķ·
    FileProvider = new PhysicalFileProvider(pathUpload),//ָʵ·
    ContentTypeProvider = new FileExtensionContentTypeProvider(new Dictionary<string, string>
                {
                    { ".apk","application/vnd.android.package-archive"},
                    { ".nupkg","application/zip"}
                })
});



app.UseAuthentication();
app.UseAuthorization();

app.MapControllers();

app.Run();
